package edu.stanford.nlp.framenet;

import javax.xml.parsers.DocumentBuilder; 
import javax.xml.parsers.DocumentBuilderFactory;  
import javax.xml.parsers.FactoryConfigurationError;  
import javax.xml.parsers.ParserConfigurationException;
 
import org.xml.sax.SAXException;  
import org.xml.sax.SAXParseException;  

import org.w3c.dom.*;

import java.io.*;
import java.util.*;

import edu.stanford.nlp.trees.*;

/**  Uses DOM libraries to read in a lexical unit xml file from FrameNet 2.
 * <p>
 * Warnings:
 *  --There are several places where messages are printed out where exceptions 
 *    should really be thrown
 *
 * @author Cindi Thompson
 */
public class FN2Reader {

  // Global value so it can be ref'd by the tree-adapter
  static Document document;

  //where we are looking in document
  private Node ptr; 
  private Vector sentences;
  private Iterator sentPtr;

  //  private Reader in;

  /** would really want this to take a Reader, but DocumentBuilder parse 
    needs a InputStream or File
    */
  public FN2Reader(File in) {
    //    this.in = in;
    
    DocumentBuilderFactory factory =
      DocumentBuilderFactory.newInstance();
    factory.setValidating(true);   //for checking against the DTD
    try {
      DocumentBuilder builder = factory.newDocumentBuilder();
      //can parse a File or InputStream or String containing URI
      document = builder.parse(in );
      ptr = document;
      sentences = new Vector();
    } catch (SAXParseException spe) {
      // Error generated by the parser
      System.out.println("\n** Parsing error"
			 + ", line " + spe.getLineNumber()
			 + ", uri " + spe.getSystemId());
      System.out.println("   " + spe.getMessage() );
      
      // Use the contained exception, if any
      Exception  x = spe;
      if (spe.getException() != null)
	x = spe.getException();
      x.printStackTrace();
      
    } catch (SAXException sxe) {
      // Error generated during parsing)
      Exception  x = sxe;
      if (sxe.getException() != null)
	x = sxe.getException();
      x.printStackTrace();
      
    } catch (ParserConfigurationException pce) {
      // Parser with specified options can't be built
      pce.printStackTrace();
      
    } catch (IOException ioe) {
      // I/O error
      ioe.printStackTrace();
    }        
  }//FN2Reader(File)

  /**
   * reads a FNSentence from the xml file.
   * Really a misnomer, the FNSentbank has been "read", this just gets one
   *
   * LEFT OFF: this basically works, now 
   * fix up main routine to 
   *   -echo the data there
   *   -allow both single xml files and a directory
   */
  //  public FNSentence readSent() throws IOException {
  public FNSentence readSent()  {

    //The first time readSent is called, the sentence list & iterator are
    // initialized.  Could do this in constructor, but for memory savings 
    // only intialize when sentences are actually requested
    if (sentences.isEmpty()) {
      int type = ptr.getNodeType();
      switch (type) {
      case Node.DOCUMENT_NODE: {
	NodeList list = ((Document)ptr).getElementsByTagName("sentence");
	for (int i=0; i<list.getLength();i++) {
	  sentences.add(list.item(i));
	}
	sentPtr = sentences.iterator();
	ptr = ((Document)ptr).getDocumentElement();
	break;
      }
      case Node.ELEMENT_NODE: {
	System.out.print("Warning, sentences still null; Element node, name: ");
	System.out.println(ptr.getNodeName());
	NodeList nodes = ptr.getChildNodes();
	if (nodes != null) {
	  Vector children = new Vector();
	  int len = nodes.getLength();
	}
	ptr = null;
	break;
      }
      default: {
	System.out.println("Another type of node");
      }
      }//switch on node type
    }//null sentences
    if (sentPtr.hasNext()) {
      Node sent = (Node)sentPtr.next();
      NodeList elementsList = sent.getChildNodes();
      FNSentence aSent = new FNSentence();
      for (int j = 0; j < elementsList.getLength(); j++){
        Node attribute = elementsList.item(j);
	String strValue;
	// this code needs cleaning up, just test for ELEMENT type
        if ( Node.ELEMENT_NODE == attribute.getNodeType()) {
	  //special handling for the sentence text
	  if (attribute.getNodeName().equals("text"))
	    aSent.setSentence(((Text)attribute.getChildNodes().item(0)).getData());
	  else
	    aSent = handleElement(aSent, (Element)attribute);
	}
      }
      return aSent;
    } else
      return null;
  }//readSent()

  //get the names and labels of each element and add the ones that 
  // we are using to thisSent
  private FNSentence handleElement(FNSentence thisSent, Element node) {
    NodeList children = node.getChildNodes();
    Node each;
    if (children != null) {
      int len = children.getLength();
      FNLayer thisLayer=new FNLayer();
      for (int i=0;i<len;i++) {
	if (Node.ELEMENT_NODE == children.item(i).getNodeType()) {
	  Element ithChild = (Element)children.item(i);
	  String nameAtt= ithChild.getAttribute("name");
	  NodeList labels = ithChild.getElementsByTagName("label");
	  if (nameAtt.equals("FE")) {//new FNRole
	    thisLayer = addNewRole(thisLayer, labels, 
				   ithChild.getAttribute("frame"), 
				   thisSent.getSentence());
	  }
	  else if (nameAtt.equals("PT") || nameAtt.equals("GF")) {
	    thisLayer = addToRole(thisLayer, nameAtt, labels);
	  }
	  else if (nameAtt.equals("Target")) {
	    thisLayer = addTarget(thisLayer, labels, thisSent.getSentence());
	  }
	}
      }
      thisSent.addLayer(thisLayer);
    }//children not null
    return thisSent;
  }//handleElement

  //create a new layer with a new FNRole
  private FNLayer addNewRole(FNLayer result, NodeList FElist, String frame, Sentence sent) {
    result.setFrame(frame);
    Element ithChild;
    String words = sent.toString();
    for (int i=0;i<FElist.getLength();i++) {
      ithChild = (Element)FElist.item(i);
      String start =ithChild.getAttribute("start");
      if (!start.equals("")) {
	Integer startInt = new Integer(start);
	Integer endInt = new Integer(ithChild.getAttribute("end"));
	String phrase = words.substring(startInt.intValue(), endInt.intValue()+1);
	result.addFERole(ithChild.getAttribute("name"),
		       phrase, startInt.intValue(), endInt.intValue()+1);
      } else {
	//null instantiation
	String itype = ithChild.getAttribute("itype");
	result.addFERole(ithChild.getAttribute("name"), itype, -1, -1);
      }

    }
    return result;
  }//addNewRole

  //Add a PT or GF (depends on type) to the role of this FNLayer
  private FNLayer addToRole(FNLayer init, String type, NodeList list) {
    Element ithChild;
    for (int i=0;i<list.getLength();i++) {
      ithChild = (Element)list.item(i);
      Integer startInt = new Integer(ithChild.getAttribute("start"));
      Integer endInt = new Integer(ithChild.getAttribute("end"));
      init.updateRole(startInt.intValue(), endInt.intValue()+1, type, 
		      ithChild.getAttribute("name"));
    }
    return init;
  }

  //There can be more than one target for phrasal verbs (eg, "cover up")
  private FNLayer addTarget(FNLayer init, NodeList list, Sentence sent) {
    String words = sent.toString();
    String targ="";
    for (int i=0;i<list.getLength();i++) {
      Element tnode = (Element)list.item(i);
      String start, end;
      if ((start=tnode.getAttribute("start")) != null) {
	if ((end=tnode.getAttribute("end")) != null) {
	  if (i>0) targ+=" ";
	  targ+=words.substring((new Integer(start)).intValue(),
				      (new Integer(end)).intValue()+1);
	  
	} else {
	  System.out.println("******NO end position");
	}
      } else {
	System.out.println("******NO start position");
      }
    }//only one element
    init.setTarget(targ);
    return init;
  }//addTarget

  //may not be needed at all
  public void close()  {
    //	in.close();
    ;
  }

  public static void main(String args[]) {

    if (args.length != 1) {
      System.err.println("Usage: java FN2Reader xml_filename");
      System.exit(1);
    }
    
    FN2Reader fr = new FN2Reader(new File(args[0]));
    
    //Now process the doc	
    Object nextS = fr.readSent();
    while ((nextS != Boolean.FALSE) && (nextS != null)){
      System.out.println("Next sentence & annotation:\n "+nextS.toString());
      nextS = fr.readSent();
    }//not empty doc
    
  }//main
}
