package edu.stanford.nlp.framenet;

import java.util.*;
import edu.stanford.nlp.trees.*;

/** An instance of a FrameNet role annotation.  Has a roleName, an
 * associated frame, a phrase type, grammatical function, and a span of
 * associated words. 
 *
 * @author Roger Levy, Cindi Thompson
 */
public class FNRole {
  private String roleName;

  private FNFrame frame;

  /** phrase type. really ought to be a property of the sentence, but
   * that's not advisable given that the FN database doesn't make it
   * so */
  private String pt;

  /** grammatical function */
  private String gf;

  private Sentence words;

  /** start and end position of role in original sentence; ought to be
    accessible only to other package classes
   */
  private int start, end;

  public FNRole() {
    this("", "");
  }

  /** Constructor for role of the given name and consisting of the given words.
   */
  public FNRole(String name, String origwords) {
    roleName = name;
    setWords(origwords);
  }

  public void setWords(String origwords) {
    StringTokenizer st = new StringTokenizer(origwords);
    Collection eachWord = new Vector();
    while (st.hasMoreTokens())
      eachWord.add(new Word(st.nextToken()));
    words = new Sentence(eachWord);
  }

  public String getRoleName() {
    return roleName;
  }

  public void setRoleName(String name) {
    roleName = name;
  }

  public FNFrame getFrame() {
    return frame;
  }

  public String getPT() {
    return pt;
  }

  public void setPT(String label) {
    pt = label;
  }

  public String getGF() {
    return gf;
  }

  public void setGF(String label) {
    gf= label;
  }

  protected void setStart(int start) {
    this.start=start;
  }

  protected void setEnd(int end) {
    this.end=end;
  }
  
  protected int getStart() {
    return start;
  }
  protected int getEnd() {
    return end;
  }

  public String toString() {
    return roleName+": \""+words.toString()+"\" PT: "+pt+" GF: "+gf+"; ";
  }
}
