package edu.stanford.nlp.framenet;

import java.util.*;
import edu.stanford.nlp.trees.*;

/** A FrameNet sentence.  A Sentence of words, and a Collection of
 * annotation layers on that sentence, where one layer of annotation
 * corresponds to one annotation <i>target</i>
 *
 * @author Roger Levy, Cindi Thompson
 */
public class FNSentence {

  private Sentence sentence;
  /** the set of annotation layers (1 layer corresponding to 1 target)
   * for the sentence */
  private Collection layers;

  /** ID from FrameNet annotation */
  private int ID;
  /** aPos from FrameNet annotation */
  private String aPos;

  public FNSentence() {
    sentence = new Sentence();
    layers = new HashSet();
  }

  public Sentence getSentence() {
    return sentence;
  }

  public void setSentence(String origwords) {
    StringTokenizer st = new StringTokenizer(origwords);
    Collection eachWord = new Vector();
    while (st.hasMoreTokens())
      eachWord.add(new Word(st.nextToken()));
    sentence = new Sentence(eachWord);
  }

  /** return an array of the layers (one per target) of the sentence. */
  public FNLayer[] getLayers() {
    FNLayer[] l = new FNLayer[layers.size()];
    int n = 0;
    for(Iterator i = layers.iterator(); i.hasNext(); ){
      l[n] = (FNLayer) i.next();
      n++;
    }
    return l;
  }//getLayers

  public void addLayer(FNLayer layer) {
    layers.add(layer);
  }//addLayer

  public String toString() {
    String ret=sentence.toString()+"\nLayers: ";
    for (Iterator iT=layers.iterator(); iT.hasNext();)
      ret += iT.next().toString();
    return ret;
  }

}
